package Html;

use strict;
use warnings;

use Exporter;
use Config qw#get_config#;
use Data::Setup qw#@SELECTS#;

@Html::EXPORT_OK = qw(escape head foot navbar get_filename get_title);
@Html::ISA = ('Exporter');

=head1 Html.pm

A set of utilities used by report modules.

=cut

return (1);

# NB: preserves all &foo; to allow inclusion of strange characters
#     returns list
sub escape
{
	my @esc = map
	{
		s/</&lt;/g;
		s/>/&gt;/g;
		s/"/&quot;/g;
		s/\s{2,}/&nbsp;/g;
		$_;
	} (@_);

	if (wantarray ())
	{
		return (@esc);
	}
	else
	{
		return (join ('', @esc));
	}
}

# generates only common header - with title and head.
sub head
{
	my ($title, $header) = @_;
	my $text;
	my $charset = get_config ('html_charset');
	my $stylesheet = get_config ('html_stylesheet');

	if (!defined ($charset) or !$charset) { $charset = 'iso-8859-1'; }
	if (!defined ($stylesheet) or !$stylesheet) { $stylesheet = 'style.css'; }
	
	$text = qq#<?xml version="1.0" encoding="$charset"?>\n#
	.	qq#<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN"\n#
	.	qq#\t"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">\n#
	.	qq#<html>\n<head>\n#;
	
	if ($stylesheet)
	{
		$text .= qq#  <link rel="stylesheet" type="text/css" #
		.	qq#href="$stylesheet" />\n#;
	}
	
	$text .= "  <title>$title</title>\n"
	.	qq#  <meta name="robots" value="noindex, nofollow" />\n#
	.	"</head>\n\n"
	.	"<body>\n";
	
	$text .= qq#<h1><img src="logo.png" /> $header</h1>\n# if $header;
	return $text;
}

sub foot
{
	my ($a, $e);
	my $text = "<hr />\n"
	.	qq#<p id="footer">Generated by <a href="$::HOMEPAGE">$::NAME $::VERSION</a>, #
	.	scalar (localtime ())
	.	"</p>\n";

	$text .= "\n</body>\n</html>\n";
	return $text;
}

sub navbar
{
	my $sel = shift;
	my $text = "<p>\n";

	if (defined ($sel) and ref ($sel))
	{
		$text .= qq#  [&nbsp;<a href="index.html">General</a>&nbsp;]\n#;
	}
	else
	{
		$text .= qq#  [&nbsp;General&nbsp;]\n#;
		$sel = '';
	}

	for (@SELECTS)
	{
		my $this_sel = $_;
		my $title = get_title ($this_sel);
		$title =~ s/\s+/&nbsp;/g;

		if ("$this_sel" eq "$sel")
		{
			$text .= qq#  [&nbsp;$title&nbsp;]\n#;
		}
		else
		{
			my $filename = get_filename ($this_sel);
			$text .= qq#  [&nbsp;<a href="$filename">$title</a>&nbsp;]\n#;
		}
	}

	$text .= "</p>\n\n";
	return ($text);
}

sub get_filename
{
	my $sel = shift;

	my $agg = $sel->[0];
	my $flds = join ('-', @{$sel->[1]});

	my %sign_names =
	(
		'=='	=> 'eq',
		'eq'	=> 'eq',
		'>='	=> 'ge',
		'<='	=> 'le',
		'!='	=> 'ne',
		'=~'	=> 're',
		'!~'	=> 'nre',
		'<'	=> 'lt',
		'>'	=> 'gt'
	);
	
	my $filename = join ('_', ($agg, 'BY', $flds));
	
	if (scalar (@{$sel->[2]}))
	{
		my @where = ();
		for (@{$sel->[2]})
		{
			my ($key, $op, $val) = @$_;
			$val =~ s/\W//g;
			push (@where, join ('-', ($key, $sign_names{$op}, $val)));
		}

		$filename .= '_WHERE_' . join ('_AND_', @where);
	}
	
	$filename .= '.html';

	return ($filename);
}

sub get_title
{
	my $sel = shift;

	my $title = ucfirst ($sel->[0]) . ' by ';
	
	my @flds = map { ucfirst ($_) } (@{$sel->[1]});
	if (scalar (@flds) > 2)
	{
		my $last = $#flds - 1;
		$title .= join (', ', @flds[0 .. $last]);
		$title .= ' and ' . $flds[-1];
	}
	else
	{
		$title .= join (' and ', @flds);
	}

	if (scalar (@{$sel->[2]}))
	{
		$title .= ' where ';
		my @wheres = map
		{
			ucfirst ($_->[0]) . ' '
			. $_->[1]
			. ' "' . $_->[2] . '"'
		} (@{$sel->[2]});

		if (scalar (@wheres) > 2)
		{
			my $last = $#wheres - 1;
			$title .= join (', ', @wheres[0 .. $last]);
			$title .= ' and ' . $wheres[-1];
		}
		else
		{
			$title .= join (' and ', @wheres);
		}
	}

	($title) = escape ($title);
	return ($title);
}
