package Parser;

# Written by David Augros <david@lightship.net>

use strict;
use warnings;
use vars qw(%DATAFIELDS);

use Parser::WebserverTools qw#%MONTH_NUMBERS#;
use Exporter;

@Parser::EXPORT_OK = qw#%DATAFIELDS parse extra#;
@Parser::ISA = ('Exporter');

our %extra = (
	total	=> 0,
	days	=> {}
);

our %severity = map
	{ $_ => 1 }
	(qw#kern user mail daemon auth syslog lpr
	news uucp cron authpriv ftp
	local0 local1 local2 local3
	local4 local5 local6 local7#);

%DATAFIELDS = (
	date		=> 'key:date',
	hour		=> 'key:hour',
	client		=> 'key:host',
	
	query		=> 'key',
	class		=> 'key',
	type		=> 'key',

	severity	=> 'key',
	category	=> 'key',
	
	requests	=> 'agg'
);

require Data::Core;
import Data::Core qw#store#;

my ($default_hour, $default_day, $default_year) = (localtime ())[2,3,5];
my $default_month = (split (m/\s+/, scalar (localtime ())))[1];
$default_year += 1900;

my $VERSION = '$Id: Bind9.pm,v 1.1 2003/03/11 18:36:51 octo Exp $';
print STDERR $/, __FILE__, ": $VERSION" if ($::DEBUG);

return (1);

sub parse
{
	my $line = shift or return undef;
	#if ($line =~ m/^(?:(\w{3}) (\d+) (\d\d)[\d:\.]+ )?(?:(\w+): )?(?:(\w+): )?client ([\d\.])#\d+: query: (\S+) (\S+) (\S+)$/)
	if ($line =~ m/^(?:(\w{3}) (\d+) (\d\d)[\d:\.]+ )?(?:(\w+): )?(?:(\w+): )?client ([\d\.]+)#\d+: query: (\S+) (\S+) (\S+)$/)
	{
		my ($month, $day, $hour) = ($default_month, $default_day, $default_hour);
		if (defined ($1) and $1)
		{
			($month, $day, $hour) = ($1, $2, $3);
		}
		my $date = sprintf ("%04u-%02u-%02u",
			$default_year, $MONTH_NUMBERS{$month}, $day);

		my $category = '*UNKNOWN*';
		my $severity = '*UNKNOWN*';
		if (defined ($4) and $4 and defined ($5) and $5)
		{
			$category = $4;
			$severity = $5;
		}
		elsif (defined ($4) and $4)
		{
			if (defined ($severity{$4})) { $severity = $4; }
			else { $category = $4; }
		}
		elsif (defined ($5) and $5)
		{
			if (defined ($severity{$5})) { $severity = $5; }
			else { $category = $5; }
		}

		my ($client, $query, $class, $type) = ($6, $7, $8, $9);

		if ($query =~ m/in-addr\.arpa$/)
		{
			my @tmp = reverse (split (m/\./, $query));
			splice (@tmp, 0, 2);

			$query = join ('.', @tmp);
		}
			
		$extra{'total'}++;
		if (!defined $extra{'days'}->{$date}) { $extra{'days'}->{$date} = 1; }

		my %combined = (
			date		=> $date,
			hour		=> $hour,
			client		=> $client,
	
			query		=> $query,
			class		=> $class,
			type		=> $type,
			
			severity	=> $severity,
			category	=> $category,

			requests	=> 1
		);

		store (\%combined);
	}
	elsif ($::DEBUG)
	{
		chomp ($line);
		print $/, __FILE__, ": Unable to parse: $line";
	}
}

sub extra
{
	my ($average, $days) = (0, 1);
	
	return (0) unless ($extra{'total'});
	
	$days = scalar (keys (%{$extra{'days'}}));
	
	$::EXTRA->{'Total requests'} = $extra{'total'};
	$::EXTRA->{'Average requests per day'} = sprintf ("%.1f", $extra{'total'} / $days);;
	$::EXTRA->{'Reporting period'} = "$days days";
}
