package Parser;

use strict;
use warnings;
use vars qw(%DATAFIELDS);

use Exporter;
use Parser::WebserverTools qw(%MONTH_NUMBERS);
use Config qw#get_config#;

@Parser::EXPORT_OK = qw(parse extra %DATAFIELDS);
@Parser::ISA = ('Exporter');

our %extra = (
	total	=> 0,
	days	=> {}
);

%DATAFIELDS = (
	host	=> 'key:host',
	user	=> 'key',
	date	=> 'key:date',
	hour	=> 'key:hour',
	tld	=> 'key',
	file	=> 'key',
	status	=> 'key:numeric',
	bytes	=> 'agg:bytes',
	requests => 'agg'
);

# This needs to be done at runtime, since Data uses Setup which relies on
# %DATAFIELDS to be defined  -octo
require Data::Core;
import Data::Core qw#store#;

my $VERSION = '$Id: Common.pm,v 1.8 2003/03/11 18:35:45 octo Exp $';
print STDERR $/, __FILE__, ": $VERSION" if ($::DEBUG);

return (1);

sub parse
{
	my $line = shift or return undef;
	if ($line =~ /^(\S+) (\S+) (\S+) \[([^\]]+)\] "([^"]+)" (\d+) (\d+|-)$/)
	{
		my ($host, $ident, $user, $date, $request, $status, $bytes) =
			($1, $2, $3, $4, $5, $6, $7);

		my ($day, $month, $year, $hour, $minute, $second) =
			$date =~ m#(\d\d)/(\w{3})/(\d{4}):(\d\d):(\d\d):(\d\d)#;
		
		$month = $MONTH_NUMBERS{$month};
		$date = sprintf("%04u-%02u-%02u", $year, $month, $day);

		my ($method, $file, $params) =
			$request =~ m#(\S+) ([^\?\s]+)\??(\S*)#
			or return (0);
		
		if (($user ne '-') and ($status >= 400) and ($status < 500))
		{
			$user = '*INVALID*';
		}

		if ($user eq '-') { $user = '*UNKNOWN*'; }
		if ($bytes eq '-') { $bytes = 0; }

		my $tld;
		if ($host =~ m/\.([a-z]{2,})$/i)
		{
			$tld = lc ($1);
		}
		else
		{
			$tld = '*UNRESOLVED*';
		}

		$extra{'total'}++;
		$extra{'days'}->{$date}++;

		my %combined = (
					'host'		=>	$host,
					'user'		=>	$user,
					'date'		=>	$date,
					'hour'		=>	$hour,
					'tld'		=>	$tld,
					'file'		=>	$file,
					'status'	=>	$status,
					'bytes'		=>	$bytes,
					'requests'	=>	1
				);
		store (\%combined);
	}
	elsif ($::DEBUG)
	{
		chomp ($line);
		print STDERR $/, __FILE__, ": Unable to parse: '$line'";
	}
}

sub extra
{
	my ($average, $days) = (0, 0);
	
	$days = scalar (keys (%{$extra{'days'}}));

	return (0) unless ($days);
	
	$average = sprintf ("%.1f", ($extra{'total'} / $days));

	$::EXTRA->{'Total requests'} = $extra{'total'};
	$::EXTRA->{'Averange requests per day'} = $average;
	$::EXTRA->{'Reporting period'} = "$days days";
}
