package Parser;

use strict;
use warnings;
use vars qw(%DATAFIELDS);

use Exporter;
use Config qw#get_config#;

@Parser::EXPORT_OK = qw#parse extra %DATAFIELDS#;

@Parser::ISA = ('Exporter');

# year 2038 unsafe
our %extra =
(
	'start'	=> (2**31 - 1),
	'end'	=> 0,
	'total'	=> 0,
	'days'	=> {}
);

%DATAFIELDS = (
	'date'		=> 'key:date',
	'hour'		=> 'key:hour',
	'client'	=> 'key:host',
	'server'	=> 'key:host',
	'peer'		=> 'key:host',
	'protocol'	=> 'key',
	'method'	=> 'key',
	'mime'		=> 'key',
	'httpstatus'	=> 'key:numeric',
	'resultcode'	=> 'key',
	'hierarchycode'	=> 'key',
	'bytes'		=> 'agg:bytes',
	'elapsed'	=> 'agg',
	'requests'	=> 'agg'
);

# This needs to be done at runtime, since Data uses Setup which relies on
# %datafields to be defined  -octo
require Data::Core;
import Data::Core qw#store#;

my $VERSION = '$Id: Squid.pm,v 1.5 2003/03/11 19:09:12 octo Exp $';
print STDERR $/, __FILE__, ": $VERSION" if ($::DEBUG);

return (1);

sub parse
{
	my $line = shift or return undef;
	#if ($line =~ m#^(\S+)\s+(\d+)\s(\S+)\s([^/]+)/(\d+)\s(\d+)\s(\S+)\s(\S+)\s(\S+)\s([^/]+)/(\S+)\s(.*)$#)
	if ($line =~ m#^(\S+)\s+(\d+) (\S+) ([^/]+)/(\d+) (\d+) (\S+) (\S+) (\S+) ([^/]+)/(\S+) (.*)$#)
	{
		my ($epoch, $duration, $client, $result_code, $http_code,
			$size, $method, $url, $ident, $hierarchy_code,
			$origin_host, $mime) = 
		($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12);

		return undef if ($url eq '*');

		my $hour  = sprintf ("%02u", (localtime ($epoch))[2]);
		my $day   = sprintf ("%02u", (localtime ($epoch))[3]);
		my $month = sprintf ("%02u", (localtime ($epoch))[4] + 1);
		my $year  = sprintf ("%04u", (localtime ($epoch))[5] + 1900);

		my $date = sprintf ("%04u-%02u-%02u", $year, $month, $day);

		$extra{'total'}++;
		$extra{'days'}->{$date}++;
		$extra{'start'} = $epoch if ($epoch < $extra{'start'});
		$extra{'end'}   = $epoch if ($epoch > $extra{'end'});

		my ($protocol, $server, $path);
		if ($url =~ m#^([^:]+)://([^:/]+)(?::\d+)?(/[^\?]*)#)
		{
			($protocol, $server, $path) = ($1, $2, $3);
		}
		elsif ($url =~ /^([\w\d\-\.]+):443$/)
		{
			($protocol, $server, $path) = ('https', $1, '');
		}
		elsif ($::DEBUG)
		{
			print STDERR $/, __FILE__, ": Unable to parse URL: $url";
			return undef;
		}
		else
		{
			return undef;
		}
		
		my %combined=(
			    'client'	    => $client,
			    'resultcode'    => uc ($result_code),
			    'httpstatus'    => $http_code,
			    'method'	    => $method,
			    'mime'	    => $mime,
			    'bytes'	    => $size,
			    'server'	    => $server,
			    'protocol'	    => uc ($protocol),
			    'hierarchycode' => uc ($hierarchy_code),
			    'peer'	    => $origin_host,
			    'date'	    => $date,
			    'hour'	    => $hour,
			    'elapsed'	    => $duration,
			    'requests'	    => 1,
			    );
		store (\%combined);
	}
	elsif ($::DEBUG)
	{
		chomp ($line);
		print STDERR $/, __FILE__, ": Unable to parse: '$line'";
	}
}

sub extra 
{
	my $start = scalar (localtime ($extra{'start'}));
	my $end   = scalar (localtime ($extra{'end'}));
	my $days  = scalar (keys (%{$extra{'days'}}));
	my $average = 0;
	if ($days) { $average = sprintf ("%.1f", ($extra{'total'} / $days)); }

	$::EXTRA->{'Total requests'} = $extra{'total'};
	$::EXTRA->{'Reporting period'} = "$days Days";
	$::EXTRA->{'Averange requests per day'} = $average;
	$::EXTRA->{'Start date'} = $start;
	$::EXTRA->{'End date'} = $end;
}
