# ncsa.pm was patched to support wn-server by M. Feenstra on 20/09/2001
package Parser;

use strict;
use warnings;
use vars qw(%DATAFIELDS);

use Exporter;
use WebserverTools qw(%MONTH_NUMBERS detect_referer detect_browser detect_os
	extract_data);

@Parser::EXPORT_OK = qw(parse extra %DATAFIELDS);
@Parser::ISA = ('Exporter');

our %extra = (
	total	=> 0,
	days	=> {}
);

%DATAFIELDS = (
	host		=> 'key:host',
	user		=> 'key',
	date		=> 'key:date',
	hour		=> 'key:hour',
	browser		=> 'key',
	os		=> 'key',
	tld		=> 'key',
	file		=> 'key',
	referer		=> 'key:url',
	status		=> 'key:numeric',
	virtualhost	=> 'key',
	bytes		=> 'agg:bytes',
	requests	=> 'agg'
);

# This needs to be done at runtime, since Data uses Setup which relies on
# %DATAFIELDS to be defined  -octo
require Data::Core;       
import Data::Core qw#store#; 

my $VERSION = '$Id: Wnserver.pm,v 1.5 2003/03/11 19:11:16 octo Exp $';
print STDERR $/, __FILE__, ": $VERSION" if ($::DEBUG);    

return (1);

sub parse
{
	my $line = shift or return undef;
#	if ($line =~ /^(\S+)\s(\S+)\s(.+?)\s\[(.+?)\]\s"(.+?)"\s(\d+)\s(\S+)\s<.*?><(.*?)>\s<(.*?)>\s<(.*?)>\s<.*?>(?:\s<(.*?)>)?$/)
	if ($line =~ /^(\S+) (\S+) (\S+) \[([^\]]+)\] "([^"]+)" (\d+) (\d+) <[^>]*><([^>]*)> <([^>]*)> <([^>]*)> <([^>]*)>$/)
	#if ($line =~ /^(\S+)\s(\S+)\s(\S+)\s\[([^\]]+)\]\s\"([^\"]+)\"\s(\d+)\s(\d+)\s\<[^\>]*\>\<([^\>]*)\>\s\<([^\>]*)\>\s\<([^\>]*)\>\s\<([^\>]*)\>$/)
	{
		my ($host, $ident, $user, $date, $request, $status,
			$bytes, $browser, $referer, $cookie, $nickname) =
		($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11);

		my ($day, $month, $year, $hour, $minute, $second) =
			$date =~ m#(\d\d)/(\w{3})/(\d{4}):(\d\d):(\d\d):(\d\d)#;

		$month = $MONTH_NUMBERS{$month};
		$date = sprintf("%04u-%02u-%02u", $year, $month, $day);

		my ($method, $file, $params) =
			$request =~ m#(\S+) ([^\?]+)\??(\S*)#
				or return (0);
		
		if (($user ne '-') and ($status >= 400) and ($status < 500))
		{
			$user = '*INVALID*';
		}

		if ($user eq '-') { $user = '*UNKNOWN*'; }
		if ($bytes eq '-') { $bytes = 0; }

		my $tld;
		if ($host =~ m/\.([a-z]{2,})$/i)
		{
			$tld = lc ($1);
		}
		else
		{       
			$tld = '*UNRESOLVED*';
		}

		my $os = detect_os ($browser);
		my $browser_name = detect_browser ($browser);
		my @search_terms = extract_data ($referer);
		if ($referer eq '-') { $referer = ''; }
		
		$extra{'total'}++;
		$extra{'days'}->{$date}++;

		if (scalar @search_terms)
		{
			print $/, __FILE__, ": Search Terms: ",
				join (' ', @search_terms)
				if ($::DEBUG & 0x1000);
			
			$extra{'search_terms'}->{$_}++ for (@search_terms);
		}
		
		my %combined = (
					'host'		=>	$host,
					'user'		=>	$user,
					'date'		=>	$date,
					'hour'		=>	$hour,
					'browser'	=>	$browser_name,
					'os'		=>	$os,
					'tld'		=>	$tld,
					'file'		=>	$file,
					'referer'	=>	$referer,
					'status'	=>	$status,
					'bytes'		=>	$bytes,
					'virtualhost'	=>	$nickname,
					'requests'	=>	1
										);
		store (\%combined);
	}
	elsif ($::DEBUG)
	{
		chomp ($line);
		print STDERR $/, __FILE__, ": Unable to parse: '$line'";
	}
}

sub extra
{
	my ($average, $days) = (0, 0);
	
	$days = scalar (keys (%{$extra{'days'}}));
	return (0) unless ($days);
	
	$average = sprintf ("%.1f", ($extra{'total'} / $days));

	$::EXTRA->{'Total requests'} = $extra{'total'};
	$::EXTRA->{'Averange requests per day'} = $average;
	$::EXTRA->{'Reporting period'} = "$days days";
	
	my @sorted_terms = sort
		{ $extra{'search_terms'}->{$b} <=> $extra{'search_terms'}->{$a} }
		(keys %{$extra{'search_terms'}});
	
	if (@sorted_terms)
	{
		my $max = $extra{'search_terms'}->{$sorted_terms[0]};
		my @scalar_terms = ();
		
		while (@sorted_terms and
			($extra{'search_terms'}->{$sorted_terms[0]} / $max) > 0.1)
		{
			$_ = shift (@sorted_terms);
			
			push (@scalar_terms,
				sprintf ("%s (%u)",
					$_, $extra{'search_terms'}->{$_})
			);
		}
		$::EXTRA->{'Search terms used'} = join ("<br />\n", @scalar_terms);

		if (@sorted_terms)
		{
			my $skipped = scalar (@sorted_terms);
			$::EXTRA->{'Search terms used'} .= "\n<br />$skipped more skipped";
		}
	}
}
