package Report;

use strict;
use warnings;

use Exporter;
use Html qw#head foot escape navbar get_filename get_title#;
use Data::Core qw#receive get_values#;
use Data::Setup qw#@SELECTS#;
use Data::Convert qw#convert#;
use Config qw#get_config#;
use Report::Core qw#$OUTPUTDIR#;
use Report::GDGraph qw#generate_graph $GRAPH_WIDTH $GRAPH_HEIGHT#;

@Report::EXPORT_OK = qw#generate#;
@Report::ISA = ('Exporter');

my $VERSION = '$Id: Combined.pm,v 1.6 2003/03/09 01:10:06 octo Exp $';
print STDERR $/, __FILE__, ": $VERSION" if ($::DEBUG);

return (1);

sub generate
{
	for (@SELECTS)
	{
		my $sel = $_;
		if (scalar (@{$sel->[1]}) == 1)
		{
			generate_1D_page ($sel);
		}
		elsif (scalar (@{$sel->[1]}) == 2)
		{
			generate_2D_page ($sel);
		}
		elsif (scalar (@{$sel->[1]}) == 3)
		{
			generate_3D_page ($sel);
		}
		else
		{ die; }
	}

	generate_index_page ();

	return (1);
}

sub generate_1D_page
{
	my $sel = shift;
	my ($key) = @{$sel->[1]};

	my $filename = get_filename ($sel);
	my $title = get_title ($sel);

	open (FH, '> ' . $OUTPUTDIR . $filename) or die ('open: ' . $!);
	
	print FH head ($title, $title);
	print FH navbar ($sel);

	print FH generate_1D_table ($sel, $key);
	
	print FH foot ();
	close (FH);
}

sub generate_2D_page
{
	my $sel = shift;
	my ($key1, $key2) = @{$sel->[1]};

	my $filename = get_filename ($sel);
	my $title = get_title ($sel);

	open (FH, '> ' . $OUTPUTDIR . $filename) or die ('open: ' . $!);
	
	print FH head ($title, $title);
	print FH navbar ($sel);

	print FH generate_2D_table ($sel, $key1, $key2);
	
	print FH foot ();
	close (FH);
}

sub generate_3D_page
{
	my $sel = shift;
	my ($key1, $key2, $key3) = @{$sel->[1]};

	my $filename = get_filename ($sel);
	my $title = get_title ($sel);

	open (FH, '> ' . $OUTPUTDIR . $filename) or die ('open: ' . $!);
	
	print FH head ($title, $title);
	print FH navbar ($sel);

	print FH generate_1D_table ($sel, $key3, 1);
	
	my @vals3 = get_values ($sel, $key3);
	
	for (sort (@vals3))
	{
		my $val3 = $_;
		print FH generate_2D_table ($sel, $key1, $key2, $key3, $val3);
	}
	
	print FH foot ();
	close (FH);
}

sub generate_index_page
{
	open (FH, '> ' . $OUTPUTDIR . 'index.html') or die ('open: ' . $!);

	print FH head ("yaala $::VERSION", "yaala $::VERSION - Index");
	print FH navbar ();

	if (scalar (keys (%$::EXTRA)))
	{
		print FH "\n<hr>\n<table>\n";
		for (keys (%$::EXTRA))
		{
			my $key = $_;
			my $val = $::EXTRA->{$key};

			print FH qq#  <tr>\n    <th class="top">$key</th>\n    <td>$val</td>\n  </tr>\n#;
		}
		print FH "</table>\n";
	}
	else
	{
		print FH "\n<!-- no \%::EXTRA -->\n";
	}
	
	print FH foot ();
	close (FH);
}

sub generate_1D_table
{
	my $sel = shift;
	my $key = shift;
	
	my $do_links = 0;
	if (@_) { $do_links = shift; }

	my $agg = $sel->[0];

	my @vals = get_values ($sel, $key);
	@vals = sort (@vals);
	
	my $grand_total = receive ($sel, {});
	
	my $text = "\n<hr>\n";

	my $graph_file = generate_graph ($sel, $key);
	if ($graph_file)
	{
		$text .= qq#<p><img src="$graph_file" width="$GRAPH_WIDTH" height="$GRAPH_HEIGHT" alt="[graph]" /></p>\n#;
	}
	
	$text .= "<table>\n  <tr>\n"
	.	'    <th colspan="3">' . ucfirst ($key) . "</th>\n  </tr>\n";

	for (@vals)
	{
		my $val = $_;
		my $sum = receive ($sel, {$key => $val});
		my $print_sum = convert ($agg, $sum);
		
		$text .= qq#  <tr>\n    <th class="subhdr">#;
		if ($do_links)
		{
			my $tmpval = $val;
			$tmpval =~ s/\W//g;
			$text .= qq(<a href="#$tmpval">);
		}
		$text .= $val;
		if ($do_links)
		{
			$text .= '</a>';
		}
		$text .= qq#</th>\n    <td>$print_sum</td>\n#
		.	"    <td>" . sprintf ("%.1f%%", 100 * $sum / $grand_total) . "</td>\n";
	}

	my $print_sum = convert ($agg, $grand_total);
	$text .= qq#  <tr>\n    <th class="subhdr">Total</td>\n#
	.	qq#    <td class="total">$print_sum</td>\n#
	.	qq#    <td class="total">100.0%</td>\n  </tr>\n#
	.	qq#</table>\n#;

	return ($text);
}

sub generate_2D_table
{
	my $sel = shift;
	my $key1 = shift;
	my $key2 = shift;
	
	my $text;
	
	my $key3 = '';
	my $val3 = '';
	if (scalar (@_) >= 2)
	{
		$key3 = shift;
		$val3 = shift;
	}

	my $agg = $sel->[0];
	
	my @vals1 = get_values ($sel, $key1);
	my @vals2 = get_values ($sel, $key2);

	my $this_is_the_first_line = 1;

	@vals1 = sort (@vals1);
	@vals2 = sort (@vals2);
	
	my $grand_total;
	{
		my $query = {};
		if ($key3 and $val3)
		{
			$query->{$key3} = $val3;
		}
		$grand_total = receive ($sel, $query);
	}

	my $target = '';
	if ($val3)
	{
		my $tmpval = $val3;
		$tmpval =~ s/\W//g;
		$text = qq#\n<hr id="$tmpval" />\n#;
	}
	else
	{
		$text = qq#\n<hr />\n#;
	}
	
	my $graph_file = generate_graph ($sel, $key1, $key3, $val3);
	if ($graph_file)
	{
		$text .= qq#<p><img src="$graph_file" width="$GRAPH_WIDTH" height="$GRAPH_HEIGHT" alt="[graph]" />\n#;
		
		$graph_file = generate_graph ($sel, $key2, $key3, $val3);
		$text .= qq#  <img src="$graph_file" width="$GRAPH_WIDTH" height="$GRAPH_HEIGHT" alt="[graph]" /></p>\n#;
	}
	
	$text .= qq#<table>\n#;
	
	if ($key3 and $val3)
	{
		$text .= "  <caption>$val3</caption>\n";
	}

	$text .= qq#  <tr>\n    <td colspan="2" rowspan="2" class="blank"><img src="logo.png" /></td>\n#
	. '    <th colspan="' . scalar (@vals2) . qq#"># . ucfirst ($key2) . "</th>\n"
	. qq#    <th rowspan="2">Total</th>\n#
	. qq#    <th rowspan="2">Percent</th>\n#
	. "  </tr>\n  <tr>\n";
	$text .= qq#    <th class="subhdr">$_</th>\n# for (@vals2);
	$text .= qq#  </tr>\n  <tr>\n    <th rowspan="# . scalar (@vals1)
	. qq#"># . ucfirst ($key1) . "</th>\n";
	
	for (@vals1)
	{
		my $val1 = $_;

		$text .= "  <tr>\n" unless ($this_is_the_first_line);
		$this_is_the_first_line = 0;

		$text .= qq#    <th class="subhdr">$val1</th>\n#;
			
		for (@vals2)
		{
			my $val2 = $_;
			
			my $query = { $key1 => $val1, $key2 => $val2 };
			if ($key3 and $val3)
			{
				$query->{$key3} = $val3;
			}
			
			my $this_val = receive ($sel, $query);
			my $print_val = convert ($agg, $this_val);

			$text .= '    <td>' . ($print_val ? $print_val : '&nbsp;') . "</td>\n";
		}

		my $query = { $key1 => $val1 };
		if ($key3 and $val3)
		{
			$query->{$key3} = $val3;
		}
		
		my $this_val = receive ($sel, $query);
		my $print_val = convert ($agg, $this_val);

		$text .= '    <td class="total">' . ($print_val ? $print_val : '&nbsp;') . "</td>\n";
		$text .= '    <td class="total">' . ($this_val ? sprintf ("%.1f%%", 100 * $this_val / $grand_total) : '&nbsp;') . "</td>\n";

		$text .= "  </tr>\n";
	}

	$text .= qq#  <tr>\n    <th colspan="2">Total</th>\n#;
	my @percentages = ();
	for (@vals2)
	{
		my $val2 = $_;

		my $query = { $key2 => $val2 };
		if ($key3 and $val3)
		{
			$query->{$key3} = $val3;
		}

		my $this_val = receive ($sel, $query);
		my $print_val = convert ($agg, $this_val);

		$text .= '    <td class="total">' . ($print_val ? $print_val : '&nbsp;') . "</td>\n";

		my $pc = ($this_val ? sprintf ("%.1f%%", 100 * $this_val / $grand_total) : '&nbsp;');
		push (@percentages, $pc);
	}

	my $print_val = convert ($agg, $grand_total);
	$text .= '    <td class="total">' . ($print_val ? $print_val : '&nbsp;') . "</td>\n"
	.	qq#    <td class="blank">&nbsp;</td>\n  </tr>\n  <tr>\n#;
	
	$text .= qq#    <th colspan="2">Percent</th>\n#;
	$text .= qq#    <td class="total">$_</td>\n# for (@percentages);
	$text .= qq#    <td class="blank" colspan="2">&nbsp;</td>\n#
	.	qq#  </tr>\n</table>\n#;
	
	return ($text);
}
