package Report::GDGraph;

use strict;
use warnings;
use vars qw#$GRAPH_WIDTH $GRAPH_HEIGHT#;

use Exporter;
use Data::Setup qw#@SELECTS#;
use Data::Core qw#get_values receive#;
use Config qw#get_config#;
use Html qw#get_filename get_title#;
use Report::Core qw#$OUTPUTDIR#;

@Report::GDGraph::EXPORT_OK = qw#generate_graph $GRAPH_WIDTH $GRAPH_HEIGHT#;
@Report::GDGraph::ISA = ('Exporter');

$GRAPH_WIDTH = 500;
$GRAPH_HEIGHT = 250;

our $HAVE_GD_GRAPH = 0;
our $MAX_VALUES = 25;

my $VERSION = '$Id: GDGraph.pm,v 1.4 2003/03/09 01:21:38 octo Exp $';
print STDERR $/, __FILE__, ": $VERSION" if ($::DEBUG);

eval "use GD::Graph::bars;";
if (!$@)
{
	$HAVE_GD_GRAPH = 1;
	print STDERR ' - GD::Graph is installed' if ($::DEBUG);
}
else
{
	print STDERR ' - GD::Graph is NOT installed' if ($::DEBUG);
}

if (get_config ('graph_height'))
{
	my $height = get_config ('graph_height');
	$height =~ s/\D//g;

	if (($height > 100) and ($height < 1000))
	{
		$GRAPH_HEIGHT = $height;
	}
}

if (get_config ('graph_width'))
{
	my $width = get_config ('graph_width');
	$width =~ s/\D//g;

	if (($width > 100) and ($width < 1000))
	{
		$GRAPH_WIDTH = $width;
		$MAX_VALUES = int ($GRAPH_WIDTH / 20);
	}
}

if ($::DEBUG & 0x100)
{
	print STDERR $/, __FILE__, ': Size: ', $GRAPH_WIDTH, 'x', $GRAPH_HEIGHT,
	"; Max number of values: $MAX_VALUES";
}

return (1);

sub generate_graph
{
	my $sel = shift;
	my $key = shift;

	my $where_key = shift;
	my $where_val = shift;

	return ('') unless ($HAVE_GD_GRAPH);

	if (!defined ($where_key) or !defined ($where_val)
			or !$where_key or !$where_val)
	{
		$where_key = '';
		$where_val = '';
	}

	my $agg = $sel->[0];
	
	my $filename = get_filename ($sel);
	{
		my $replacement = "__$key";
		if ($where_key)
		{
			$replacement .= "__$where_key" . "_$where_val";
		}
		$replacement =~ s/\W+/_/g;
		$replacement .= '.png';

		$filename =~ s/\.html$/$replacement/;
	}
	
	my @key_values = get_values ($sel, $key);
	@key_values = sort (@key_values);

	my @agg_values = ();
	for (@key_values)
	{
		my %query = ($key => $_);
		if ($where_key) { $query{$where_key} = $where_val; }
		
		my $sum = receive ($sel, \%query);
		push (@agg_values, $sum);
	}

	if (scalar (@key_values) > $MAX_VALUES)
	{
		discard_values (\@key_values, \@agg_values);
	}

	for (@key_values)
	{
		next if (length ($_) < 20);

		substr ($_, 17) = ' ..';
	}

	my @data_set = (\@key_values, \@agg_values);

	my $title = ucfirst ($agg) . ' by ' . ucfirst ($key);
	if ($where_val) { $title .= ' for ' . $where_val; }
	
	print STDERR $/, __FILE__, qq#: Generating image "$title" [$filename]#
	if ($::DEBUG & 0x100);
	
	my $graph = GD::Graph::bars->new ($GRAPH_WIDTH, $GRAPH_HEIGHT);
	$graph->set
	(
		title		=> $title,
		x_label		=> ucfirst ($key),
		y_label		=> ucfirst ($agg),
		
		x_labels_vertical => 1,
		x_label_position  => 1,
		long_ticks	=> 1,

#		logo		=> 'reports/logo.png',
		transparent	=> 1,
		shadow_depth	=> 2,

		fgclr		=> 'lgray',
		bgclr		=> 'white',
		dclrs		=> [ qw(lgray gray dgray) ],
		borderclrs	=> [ qw(black black white) ],
		shadowclr	=> 'gray',
		labelclr	=> 'black',
		axislabelclr	=> 'black',
		legendclr	=> 'black',
		valuesclr	=> 'black',
		textclr		=> 'black'
	);
	
	# FIXME
	open (IMG, ">  $OUTPUTDIR$filename") or die ('open: ' . $!);
	binmode IMG;
	print IMG $graph->plot(\@data_set)->png;
	close IMG;

	return ($filename);
}

sub discard_values
{
	my $key_array = shift;
	my $val_array = shift;

	my @orig_sort = @$key_array;
	my $num_values = scalar (@$key_array);

	my %tmp_hash = ();
	
	my $i;
	for ($i = 0; $i < $num_values; $i++)
	{
		my $key = shift (@$key_array);
		my $val = shift (@$val_array);

		$tmp_hash{$key} = $val;
	}

	my @small_sorted = sort { $tmp_hash{$b} <=> $tmp_hash{$a} } (keys (%tmp_hash));

	for ($i = 0; $i < $MAX_VALUES; $i++)
	{
		shift (@small_sorted);
	}

	my $other = 0;
	for (@orig_sort)
	{
		my $this_key = $_;
		if (grep { $_ eq $this_key } (@small_sorted))
		{
			$other += $tmp_hash{$this_key};
		}
		else
		{
			push (@$key_array, $this_key);
			push (@$val_array, $tmp_hash{$this_key});
		}
	}

	push (@$key_array, '*OTHER*');
	push (@$val_array, $other);
}
